/*= ASYC II ===============================================================*/
/* LabWindows Instrument Driver for MX240, MX2040.                         */
/* Original Release: November ,15th,1996                                   */
/* By: O. LEMAIRE                                                          */
/* Originally written in C                                                 */
/* Modification History: None                                              */
/*=========================================================================*/

#include "utility.h"
#include "ansi_c.h"
#include "rs232.h"
#include "formatio.h"
#include "mx240.h"

/*= STATIC VARIABLES ======================================================*/
static int  port;                    /* number of the port opened                 */
static int  rscnt;                   /* number of bytes read or written on RS232  */
static int  mx240_err;               /* error variable for the instrument module  */
static int	mx240_instrument;        /* model of the instrument                   */
static double mx240_version;	     /* version of software instrument		      */
static int  mx240_commut_hard;       /* position of the rotary switch (hardware)  */
static int  mx240_commut_soft;       /* position of the rotary switch (software)  */
static int  mx240_display1;          /* configuration of measurement (display1)   */
static int  mx240_display2;          /* configuration of measurement (display2)   */
static int  mx240_range1;            /* range of the measurement (display1)       */
static int  mx240_range2;            /* range of the measurement (display2)       */
static int  mx240_button;            /* last button pressed                       */
static int  mx240_talk;              /* flag TALKER mode (Off/On)                 */
static int  mx240_remote;            /* flag REMOTE mode (Off/On)                 */
static int  mx240_surv;              /* flag SURV mode (Off/On)                   */
static int  mx240_hold;              /* flag HOLD mode (Off/On)                   */
static int  mx240_store;             /* flag DataLogSTORE mode (Off/On)           */
static int  mx240_dataout;           /* flag DataLogOUT mode (Off/On)             */
static int  mx240_menu;              /* flag MENU mode (Off/On)                   */
static int  mx240_freq;              /* frequency of DataLog                      */
static int  mx240_prof;              /* number of DataLog elements                */
static int  mx240_unit;        	     /* type of measurement (int)                 */
static char mx240_smeasure[20];      /* measurement (string)                      */
static double mx240_dmeasure;        /* measurement (double)                      */
static double mx240_coef;            /* coef of the measurement                   */
static double mx240_buff[525];		 /* DatalogOut								  */


/*= UTILITY ROUTINES ======================================================*/
int mx240_invalid_integer_range (int val, int min, int max, int err_code);
int mx240_invalid_real_range (double val, double min, double max, int err_code);
int mx240_device_closed (void);
int mx240_read_data (char *buf, int cnt, int term);
int mx240_write_data (char *buf, int cnt);
int mx240_send (char *buf, int cnt); /* write data and wait ACK from CLAMP */
int mx240_decode (void); 


/*=========================================================================*/
/* Function: init                                                          */
/* Purpose:  This function opens a com port for the instrument module,     */
/*           and reads the setup of the multimeter (eeprom).               */
/*=========================================================================*/
int mx240_init (int portcom)
	{
	int i;
	double x;
	
/*  check for invalid arguments */
    if (mx240_invalid_integer_range (portcom, 1, 8, -1) != 0)
       return mx240_err;

/*  Open a com port for the instrument module.  If an error occurs, set */
    OpenComConfig (portcom, "",1200,0,8,1,512,512);
    if(rs232err!=0)
    	{
        mx240_err=rs232err+400;
        return mx240_err;
    	}					  

/*  Set the timeout value for the instrument. If an error occurs, set   */
/*  mx240_err = 239 "Error in configuring timeout".                     */
    SetComTime(portcom,1.0);
    if(rs232err!=0)
    	{
        mx240_err=239;
        return mx240_err;
    	}

/*  Set port to the number of the port just opened.                     */
	port=portcom;		 

/*  REMOTE Mode actived */
    if(mx240_send(">\r",2)!=0) 
    	return mx240_err;

/*  Check for the status of the instrument */
    if (mx240_status(&i,&x,&i,&i,&i,&i,&i)!=0) 
    	return mx240_err;

	return mx240_err;
	}

/*=========================================================================*/
/* Function: Voltage                                                       */
/* Purpose:  This function switch the instrument to voltage measurement    */
/*=========================================================================*/
int mx240_voltage (int func1, int range, int func2)
	{
	int i;
	double x;
	char s[10];

/*  Check for device closed */
    if (mx240_device_closed()) 
    	return mx240_err;

/*  Check for invalid arguments */
    if (mx240_invalid_integer_range (func1, 0, 3, -1) != 0)
       return mx240_err;

/*  Check for invalid arguments */
    if ((mx240_invalid_integer_range (range, 1, 8, -2) != 0) ||
    	(mx240_invalid_integer_range (range, 3, 7, -2) == 0))
       return mx240_err;

/*  Check for invalid arguments */
    if ((mx240_invalid_integer_range (func2, 0, 10, -3) != 0) ||
    	(mx240_invalid_integer_range (func2, 2,  3, -3) == 0) ||
    	(mx240_invalid_integer_range (func2, 5,  6, -3) == 0))
       return mx240_err;

/*  Check for the status of the instrument */
    if (mx240_status(&i,&x,&i,&i,&i,&i,&i)!=0) 
    	return mx240_err;

/*  Check for the position of the rotary switch    */
	if (mx240_commut_hard == OHM)
		{
		mx240_err = 301;
		return mx240_err;
		}
		
/*  Set rotary switch */
    Fmt(s,"%s<202\r");
    if(mx240_send(s,4)!=0) 
    	return mx240_err;
	
/*  Check the Surveillance Mode */
    if(mx240_surv!=0 && mx240_commut_soft==VOLT)
    	{
		mx240_err = 304;
		return mx240_err;
		}
		
/*  Set first function */
    Fmt(s,"%s<21%i\r",func1);
   	if(mx240_send(s,4)!=0) 
   		return mx240_err;
	
/*  Set range */
    if(func1!=3)
    	{
    	Fmt(s,"%s<22%i\r",range);
   		if(mx240_send(s,4)!=0) 
   			return mx240_err;
   		}
	
/*  Set second function */
    Fmt(s,"%s<23%c\r",0x30+func2);
   	if(mx240_send(s,4)!=0) 
   		return mx240_err;
	
	return mx240_err;
	}

/*=========================================================================*/
/* Function: Current                                                       */
/* Purpose:  This function switch the instrument to current measurement    */
/*=========================================================================*/
int mx240_current (int func1, int range, int func2)
	{
	int i;
	double x;
	char s[10];

/*  Check for device closed */
    if (mx240_device_closed()) 
    	return mx240_err;

/*  Check for invalid arguments */
    if (mx240_invalid_integer_range (func1, 0, 3, -1) != 0)
       return mx240_err;

/*  Check for invalid arguments */
    if ((mx240_invalid_integer_range (range, 1, 8, -2) != 0) ||
    	(mx240_invalid_integer_range (range, 3, 7, -2) == 0))
       return mx240_err;

/*  Check for invalid arguments */
    if ((mx240_invalid_integer_range (func2, 0, 8, -3) != 0) ||
    	(mx240_invalid_integer_range (func2, 2, 3, -3) == 0))
       return mx240_err;

/*  Check for the status of the instrument */
    if (mx240_status(&i,&x,&i,&i,&i,&i,&i)!=0) 
    	return mx240_err;

/*  Check for the position of the rotary switch    */
	if (mx240_commut_hard == OHM)
		{
		mx240_err = 301;
		return mx240_err;
		}

/*  Set rotary switch */
    Fmt(s,"%s<201\r");
    if(mx240_send(s,4)!=0) 
    	return mx240_err;
	
/*  Check the Surveillance Mode */
    if(mx240_surv!=0 && mx240_commut_soft==AMP)
    	{
		mx240_err = 304;
		return mx240_err;
		}
		
/*  Set first function */
    Fmt(s,"%s<21%i\r",func1);
   	if(mx240_send(s,4)!=0) 
   		return mx240_err;

/*  Set range */
    if(func1!=3)
    	{
	    Fmt(s,"%s<22%i\r",range);
    	if(mx240_send(s,4)!=0) 
    		return mx240_err;
    	}
	
/*  Set second function */
	    Fmt(s,"%s<23%i\r",func2);
	    if(mx240_send(s,4)!=0) 
	    	return mx240_err;
	
	return mx240_err;
	}

/*=========================================================================*/
/* Function: Power (VA,W,var)                                              */
/* Purpose:  This function switch the instrument to power measurement      */
/*=========================================================================*/
int mx240_power (int unity, int function, int range, int compen, int adjust, double reference)
	{
	int i;
	double x;
	char s[10],chaine[10];

/*  Check for device closed */
    if (mx240_device_closed()) 
    	return mx240_err;

/*  Check for invalid arguments */
    if ((mx240_invalid_integer_range (unity, 3, 5, -1) != 0))
       return mx240_err;

/*  Check for invalid arguments */
    if ((mx240_invalid_integer_range (function, 0, 8, -3) != 0) ||
    	(mx240_invalid_integer_range (function, 5, 6, -3) == 0) ||
    	(function == 1))
       return mx240_err;

/*  Check for invalid arguments */
    if ((mx240_invalid_integer_range (range, 1, 8, -2) != 0) ||
    	(mx240_invalid_integer_range (range, 4, 7, -2) == 0))
       return mx240_err;

/*  Check for invalid arguments */
    if ((mx240_invalid_integer_range (compen, 0, 1, -4) != 0))
       return mx240_err;

/*  Check for invalid arguments */
    if ((mx240_invalid_integer_range (adjust, 0, 1, -5) != 0))
       return mx240_err;

/*  Check for invalid arguments */
    if ((mx240_invalid_real_range (reference, 0.0, 99999.0, -6) != 0))
       return mx240_err;

/*  Check compatibility of parameters             */
/*  Error 302 : Two arguments are incompatible    */
    if ((compen == 1 && function != 3) || (adjust == 1 && compen != 1))
    	{
       	mx240_err=302;
       	return mx240_err;
       	}

/*  Check for the status of the instrument */
    if (mx240_status(&i,&x,&i,&i,&i,&i,&i)!=0) 
    	return mx240_err;

/*  Check for the position of the rotary switch    */
	if (mx240_commut_hard == OHM)
		{
		mx240_err = 301;
		return mx240_err;
		}
		
/*  Set rotary switch */
    Fmt(s,"%s<20%i\r",unity);
    if(mx240_send(s,4)!=0) 
    	return mx240_err;
	
/*  Set second function */
    Fmt(s,"%s<23%i\r",function);
    if(mx240_send(s,4)!=0) 
    	return mx240_err;
	
/*  Set range */
    Fmt(s,"%s<22%i\r",range);
    if(mx240_send(s,4)!=0) 
    	return mx240_err;
	
/*  Set first function */
	Fmt(s,"%s<21%i\r",4+compen*4);
   	if(mx240_send(s,4)!=0) 
   		return mx240_err;
	
/*  Set value of afjust capacity */
	if(adjust==1)
		{
		Fmt(chaine,"%s<%x[w5p0]",(int)(reference*10));
		StringUpperCase(chaine);
		for(i=0;i<6;i++)
			if(chaine[i]>0x39) 
				chaine[i]=chaine[i]-7;
		
		Fmt(s,"%s<49%c%c%c%c%c\r",chaine[4],chaine[3],chaine[2],chaine[1],chaine[0]);
	   	if(mx240_send(s,8)!=0) 
	   		return mx240_err;
	   	}
	
	return mx240_err;
	}

/*=========================================================================*/
/* Function: Energy (Wh)                                                   */
/* Purpose:  This function switch the instrument to energy (Wh) measurement*/
/*=========================================================================*/
int mx240_energy (int function, int range)
	{
	int i;
	double x;
	char s[10];

/*  Check for device closed */
    if (mx240_device_closed()) 
    	return mx240_err;

/*  Check for invalid arguments */
    if (function != 0 && function != 4 && function != 8)
       {
       mx240_err = -3;
       return mx240_err;
       }

/*  Check for invalid arguments */
    if ((mx240_invalid_integer_range (range, 1, 8, -2) != 0) ||
    	(mx240_invalid_integer_range (range, 4, 7, -2) == 0))
       return mx240_err;

/*  Check for the status of the instrument */
    if (mx240_status(&i,&x,&i,&i,&i,&i,&i)!=0) 
    	return mx240_err;

/*  Check for the position of the rotary switch    */
	if (mx240_commut_hard == OHM)
		{
		mx240_err = 301;
		return mx240_err;
		}
		
/*  Set rotary switch */
    Fmt(s,"%s<206\r");
    if(mx240_send(s,4)!=0) 
    	return mx240_err;
	
/*  Set second function */
    Fmt(s,"%s<23%i\r",function);
    if(mx240_send(s,4)!=0) 
    	return mx240_err;
	
/*  Set range */
    Fmt(s,"%s<22%i\r",range);
    if(mx240_send(s,4)!=0) 
    	return mx240_err;
	
	return mx240_err;
	}

/*=========================================================================*/
/* Function: Resistance                                                    */
/* Purpose:  This function switch the instrument to resistance measurement */
/*=========================================================================*/
int mx240_resistance (int range,int cont)
	{
	int i;
	double x;
	char s[10];
	
/*  Check for device closed */
    if (mx240_device_closed()) 
    	return mx240_err;

/*  Check for invalid arguments */
    if ((mx240_invalid_integer_range (range, 1, 8, -2) != 0) ||
    	(mx240_invalid_integer_range (range, 3, 7, -2) == 0))
       return mx240_err;

/*  Check for invalid arguments */
    if (mx240_invalid_integer_range (cont, 0, 1, -3) != 0)
       return mx240_err;

/*  Check for the status of the instrument */
    if (mx240_status(&i,&x,&i,&i,&i,&i,&i)!=0) 
    	return mx240_err;

/*  Check for the position of the rotary switch    */
	if (mx240_commut_hard != OHM)
		{
		mx240_err = 301;
		return mx240_err;
		}
		
/*  Set rotary switch */
    Fmt(s,"%s<207\r");
    if(mx240_send(s,4)!=0) 
    	return mx240_err;
	
/*  Set first function */
	Fmt(s,"%s<21%c\r",0x34+cont*8);
   	if(mx240_send(s,4)!=0) 
   		return mx240_err;
	
/*  Set range */
    if(cont==0)
    	{
    	Fmt(s,"%s<22%i\r",range);
    	if(mx240_send(s,4)!=0) 
    		return mx240_err;
    	}

	return mx240_err;
	}

/*=========================================================================*/
/* Function: Timer                                                         */
/* Purpose:  This function adjust the timer value for TALKER and DataLog   */
/*           mode                                                          */
/*=========================================================================*/
int mx240_timecos (int timer, double cosinus)
	{
	char s[20],chaine[20];
	double x;
	int i;

/*  Check for device closed */
    if (mx240_device_closed()) 
    	return mx240_err;

/*  Check for invalid arguments */
    if (mx240_invalid_integer_range (timer, 0, 13, -1) != 0)
       return mx240_err;

/*  Check for invalid arguments */
    if (mx240_invalid_real_range (cosinus, 0.7, 1.0, -2) != 0)
       return mx240_err;

/*  Check for the status of the instrument */
    if (mx240_status(&i,&x,&i,&i,&i,&i,&i)!=0) 
    	return mx240_err;

/*  Set timer value */
    Fmt(s,"%s<40%c\r",0x30+timer);
    if(mx240_send(s,4)!=0) 
    	return mx240_err;
	
/*  Set cos consigne value */
	Fmt(chaine,"%s<%f",cosinus);
    Fmt(s,"%s<48%c%c\r",chaine[2],chaine[3]);
    if(mx240_send(s,5)!=0) 
    	return mx240_err;

	return mx240_err;
	}

/*=========================================================================*/
/* Function: Talker                                                        */
/* Purpose:  This function switch the TALKER mode to Out or not            */
/*=========================================================================*/
int mx240_talker (int talker)
	{
	char s[10];
	int i;
	double x;

/*  Check for device closed */
    if (mx240_device_closed()) 
    	return mx240_err;    

/*  Check for invalid arguments */
    if (mx240_invalid_integer_range (talker, 0, 1, -1) != 0)
       return mx240_err;

/*  Stop Talker Mode */
	if(talker==0)
		{
		Fmt(s,"%s<%c",0x1B);
	    if(mx240_write_data(s,1)!=0) 
	    	return mx240_err;
		Fmt(s,"%s<410\r");
	    if(mx240_send(s,4)!=0) 
	    	return mx240_err;
		}

/*  Set talker mode */
   	else 						  
		{
		Fmt(s,"%s<411\r");
	    if(mx240_send(s,4)!=0) 
	    	return mx240_err;
		}
		
	return mx240_err;
	}

/*=========================================================================*/
/* Function: DataLogStore                                                  */
/* Purpose:  This function switch the DataLogStore mode to Out or not      */
/*=========================================================================*/
int mx240_datalogstore (int datalog)
	{
	char s[10];
	double x;
	int i;

/*  Check for device closed */
    if (mx240_device_closed()) 
    	return mx240_err;

/*  Check for invalid arguments */
    if (mx240_invalid_integer_range (datalog, 0, 1, -1) != 0)
       return mx240_err;

/*  Check for the status of the instrument */
    if (mx240_status(&i,&x,&i,&i,&i,&i,&i)!=0) 
    	return mx240_err;

/*  Set DataLogStore mode */
	if(mx240_store != datalog)
		{
	    Fmt(s,"%s<42%i\r",datalog);
	    if(mx240_send(s,4)!=0) 
	    	return mx240_err;
		}
		
	return mx240_err;
	}

/*=========================================================================*/
/* Function: Surveillance                                                  */
/* Purpose:  This function switch the SURV mode 					       */
/*=========================================================================*/
int mx240_surveillance (int surv)
	{
	char s[10];
	double x;
	int i;

/*  Check for device closed */
    if (mx240_device_closed()) 
    	return mx240_err;

/*  Check for invalid arguments */
    if (mx240_invalid_integer_range (surv, 0, 1, -1) != 0)
       return mx240_err;

/*  Check for the status of the instrument */
    if (mx240_status(&i,&x,&i,&i,&i,&i,&i)!=0) 
    	return mx240_err;

/*  Set surveillance mode */
	if(mx240_surv != surv)
		{
	    Fmt(s,"%s<46%i\r",surv);
	    if(mx240_send(s,4)!=0) 
	    	return mx240_err;
		}
		
	return mx240_err;
	}

/*=========================================================================*/
/* Function: Hold		                                                   */
/* Purpose:  This function switch the HOLD mode 					       */
/*=========================================================================*/
int mx240_mem (int hold)
	{
	char s[10];
	double x;
	int i;

/*  Check for device closed */
    if (mx240_device_closed()) 
    	return mx240_err;

/*  Check for invalid arguments */
    if (mx240_invalid_integer_range (hold, 0, 1, -1) != 0)
       return mx240_err;

/*  Check for the status of the instrument */
    if (mx240_status(&i,&x,&i,&i,&i,&i,&i)!=0) 
    	return mx240_err;

/*  Set Hold mode */
	if(mx240_hold != hold)
		{
		Fmt(s,"%s<47%i\r",hold);
    	if(mx240_send(s,4)!=0) 
    		return mx240_err;
    	}

	return mx240_err;
	}

/*=========================================================================*/
/* Function: LCD                                                           */
/* Purpose:  This function switch the lcd to full display or not           */
/*=========================================================================*/
int mx240_lcd (int control)
	{
    char s[20];
	double x;
    int i;

/*  Check for device closed */
    if (mx240_device_closed()) 
    	return mx240_err;

/*  Check for invalid arguments */
    if (mx240_invalid_integer_range (control, 0, 1, -1) != 0)
       return mx240_err;

/*  Check for the status of the instrument */
    if (mx240_status(&i,&x,&i,&i,&i,&i,&i)!=0) 
    	return mx240_err;

/*  Set lcd mode */
    Fmt(s,"%s<37%i\r",control);
    if(mx240_send(s,4)!=0) 
    	return mx240_err;
    
    return mx240_err;
	}

/*=========================================================================*/
/* Function: Status                                                        */
/* Purpose:  This function checks the status of the instrument and         */
/*           initializes static variables to give informations to others   */
/*           functions.                                                    */
/*=========================================================================*/
int mx240_status (int *instrument, double *version, int *rotaryswitchhard,
				  int *rotaryswitchsoft, int *display1, int *display2, int *lastButton)
	{
    char chaine[20];

/*  Check for device closed */
    if (mx240_device_closed()) 
    	return mx240_err;

    FillBytes (chaine,  0, 15, 0);

/*  Ask for status */
    mx240_err=0;
    if(mx240_write_data("35\r",3)!=0)
    	return mx240_err;
    if(mx240_read_data(chaine,15,13)!=0)
    	return mx240_err;

/* Read the reference instrument */
	mx240_instrument=(chaine[0]-0x30)&0x04;
    if(mx240_instrument!=0 && mx240_instrument!=4)  
    	{
        mx240_err=223;
        return mx240_err;
    	}
	
/*  Read the version of Software Instrument */
    mx240_version=10*(chaine[1]-0x30)+chaine[2]-0x30+0.1*(chaine[3]-0x30)+0.01*(chaine[4]-0x30);

/*  Read the rotary switch position (0..7) */
    mx240_commut_hard=(chaine[5]-0x30)&0x07;
    mx240_commut_soft=(chaine[6]-0x30)&0x07;

/*  Read the configuration of the measurement (first display) */
    mx240_display1=chaine[7]-0x30;
	
/*  Read the configuration of the measurement (second display) */
    mx240_display2=chaine[8]-0x30;
	
/*  Read the range of the measurement (first display) */
    mx240_range1=chaine[9]-0x30;
	
/*  Read the range of the measurement (second display) */
    mx240_range2=chaine[10]-0x30;

/*  Read the last button pressed */
    mx240_button=chaine[11]-0x30;
	
/*  Read HOLD,SURV,REMOTE and TALKER mode */
    mx240_talk    =  chaine[12]&0x01;
    mx240_remote  = (chaine[12]>>1)&0x01;
    mx240_surv    = (chaine[12]>>2)&0x01;
    mx240_hold    = (chaine[12]>>3)&0x01;

/*  Read MENU,DATALOGOUT and DATALOGSTORE mode */
    mx240_menu    = (chaine[13]>>1)&0x01;
    mx240_dataout = (chaine[13]>>2)&0x01;
    mx240_store   = (chaine[13]>>3)&0x01;

	*instrument       = mx240_instrument;
	*version          = mx240_version;
	*rotaryswitchhard = mx240_commut_hard;
	*rotaryswitchsoft = mx240_commut_soft;
	*display1	      = mx240_commut_soft * 10 + mx240_display1;
	if(*display1==38 || *display1==48 || *display1==58) *display1=30;
	
	if(mx240_display2>7) *display2 = mx240_display2+2;
	else if(mx240_display2>3) *display2 = mx240_display2+16;
	else if(mx240_display2==2) *display2 = 30;
	else *display2 = mx240_display2;
	
    *lastButton	  = mx240_button;
	
	return mx240_err;
	}

/*=========================================================================*/
/* Function: Parameters                                                    */
/* Purpose:  This function reads parameters of Datalog                     */
/*=========================================================================*/
int mx240_param (int *freq, int *prof)
	{
    char chaine[20];

/*  Check for device closed */
    if (mx240_device_closed()) 
    	return mx240_err;

    FillBytes (chaine,  0, 15, 0);

/*  Ask for Frequency of DataLog */
    mx240_err=0;
    if(mx240_write_data("32\r",3)!=0)
    	return mx240_err;
    if(mx240_read_data(chaine,3,13)!=0)
    	return mx240_err;

/* Read the Frequency */
	if(chaine[1]=='0')
		{
		if(chaine[0]=='5') mx240_freq = 3;
		else mx240_freq = chaine[0] - 0x30;
		}
	else if(chaine[0]=='0') mx240_freq = 3 + chaine[1] - 0x30;
	else if(chaine[1]==':') mx240_freq = 8 + chaine[0] - 0x30;
	
/* Ask for number of DataLog elements */
    mx240_err=0;
    if(mx240_write_data("33\r",3)!=0)
    	return mx240_err;
    if(mx240_read_data(chaine,4,13)!=0)
    	return mx240_err;

/* Read the number */
	mx240_prof = (chaine[2]-0x30)*16*16 + (chaine[1]-0x30)*16 + chaine[0]-0x30;

    *freq	  = mx240_freq;
    *prof	  = mx240_prof;
	
	return mx240_err;
	}

/*=========================================================================*/
/* Function: Data                                                          */
/* Purpose:  This function configure the instrument for MIN, MAX, AVG, PEAK*/
/*           first display, second display ,datalog address or all datalog */
/*           addresses measurement.                                        */
/*           The result is in mx240_dmeasurement, mx240_smeasurement       */
/*           and mx240_unit.        									   */
/*=========================================================================*/
int mx240_data (int measure, int address, double *result, int *type)
	{
    char s[25],chaine[20];
	double x;
    int i;

/*  Check for device closed */
    if (mx240_device_closed()) 
    	return mx240_err;

/*  Check for invalid arguments */
    if (mx240_invalid_integer_range (measure, 0, 7, -1) != 0)
       return mx240_err;

/*  Check for invalid arguments */
    if (mx240_invalid_integer_range (address, 1, 512, -1) != 0)
       return mx240_err;

/*  Check for the status of the instrument */
    if (mx240_status(&i,&x,&i,&i,&i,&i,&i)!=0) 
    	return mx240_err;

    switch(measure)  
    	{
    /* MIN, MAX, AVG or PEAK measurement */
        case 0:
        case 1:
		case 2:
        case 3: 
        	Fmt(s,"%s<38%i\r",measure);
            if(mx240_write_data(s,4)!=0)    
            	return mx240_err;
            break;
    /* First or second measurement */
        case 4:
        case 5:
        	Fmt(s,"%s<3%i\r",measure-4);
            if(mx240_write_data(s,3)!=0)    
            	return mx240_err;
        	break;
	/* Read DataLog value */
        case 6:
		    Fmt(chaine,"%s<%x[w3p0]",address);
		    StringUpperCase(chaine);
		    Fmt(s,"%s<43%c%c%c\r",chaine[2],chaine[1],chaine[0]);
		    if(mx240_write_data(s,6)!=0) 
		    	return mx240_err;
        	break;
	/* Read all DataLog values */
        case 7:
			if (mx240_param(&i, &i)!=0) 
		    	return mx240_err;
		    Fmt(s,"%s<44\r");
			if(mx240_write_data(s,3)!=0) 
			   	return mx240_err;
			Delay(0.2);
		    for(i=0;i<mx240_prof;i++)
		    	{
				if(mx240_read_data (s, 21, 13)!=0)
					return mx240_err; 
				Scan(s,"%s[i5]>%f",&mx240_buff[i]);
				if(s[11]=='k')
					mx240_buff[i]=mx240_buff[i]*1e3;
				}
        	break;
        }
        
    /* Read Measurement */
    if(measure<7)
    	{
    	if(mx240_decode()!=0)
       		return mx240_err;
   
    	*result = mx240_dmeasure;
    	*type   = mx240_unit;
    	}

	return mx240_err;
	}
	
/*=========================================================================*/
/* Function: Close                                                         */
/* Purpose:  This function closes the port for the instrument module and   */
/*           sets the port to zero.                                        */
/*=========================================================================*/
int mx240_close (void)
	{
	mx240_err=0;

/*  Check for device closed */
    if (mx240_device_closed())
    	return mx240_err;

/*  REMOTE Mode desactived */
    if(mx240_send("?\r",2)!=0) 
    	return mx240_err;

/*  Wait for the end of the last send */
    while (GetOutQLen (port)!=0) {}

/*  Close the COM port. If error, set mx240_err = 221 */
    CloseCom (port);
    if (rs232err != 0) 
    	{
        mx240_err = 221;
        return mx240_err;
    	}

    port = 0;
	return mx240_err;
	}


/*=========================================================================*/
/* Function: Invalid Integer Range                                         */
/* Purpose:  This function checks an integer to see if it lies between a   */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int mx240_invalid_integer_range (int val, int min, int max, int err_code)
{
  if ((val < min) || (val > max)) {
    mx240_err = err_code;
    return -1;
  }
  return 0;
}


/*=========================================================================*/
/* Function: Invalid Real Range                                            */
/* Purpose:  This function checks a real number to see if it lies between  */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int mx240_invalid_real_range (double val, double min, double max, int err_code)
{
  if ((val < min) || (val > max)) {
    mx240_err = err_code;
    return -1;
  }
  return 0;
}


/*=========================================================================*/
/* Function: Read Data                                                     */
/* Purpose:  This function reads a buffer of data from the instrument. The */
/*           return value is equal to the global error variable.           */
/*=========================================================================*/
int mx240_read_data (char *buf, int cnt, int term)
{
    rscnt = ComRdTerm(port, buf, cnt, term);
    if (rs232err != 0)
        mx240_err = 231;
    else
        mx240_err = 0;
    return mx240_err;
}


/*=========================================================================*/
/* Function: Write Data                                                    */
/* Purpose:  This function writes a buffer of data to the instrument. The  */
/*           return value is equal to the global error variable.           */
/*=========================================================================*/
int mx240_write_data(char *buf,int cnt)
{
    /* This delay is necessary for the RS232 ASYC2 hardware */
    Delay(0.2);
    FlushInQ(port);
    FlushOutQ(port);
    rscnt = ComWrt(port,buf,cnt);
    if (rs232err !=0)
        mx240_err=230;
    else
        mx240_err=0;
    return mx240_err;
}


/*=========================================================================*/
/* Function: Send                                                          */
/* Purpose:  This function writes a buffer of data to the instrument and   */
/*           waits for an acknoledge.                                      */
/*=========================================================================*/
int mx240_send(char *buf,int cnt)
	{
	int i;
	
    /* This delay is necessary for the RS232 ASYC2 hardware */
    Delay(0.1);
    FlushInQ(port);
    FlushOutQ(port);
    rscnt = ComWrt(port,buf,cnt);
    mx240_err=0;
    if (rs232err !=0)
        {
        mx240_err=230;
        return mx240_err;
        }
    /* Wait for ACK (if Timeout return 240 */
    if((i=ComRdByte(port))!=ACK) mx240_err=240;
    return mx240_err;
}


/*=========================================================================*/
/* Function: Decode                                                        */
/* Purpose:  This function reads the measurement from the RS232 and       */
/*           initializes mx240_dmesure, mx240_smeasure                     */
/*=========================================================================*/
int mx240_decode(void)
{
    char s[25],chaine[25];

/*  Read the last measurement */
    FillBytes (s, 0, 16, 0);
    if (mx240_read_data(s,16,13)!=0) return mx240_err;

/*  Test if the measurement is overflow */
    if (s[1]=='X' || s[1]==' ') {
       Fmt(mx240_smeasure,"%s< ");
       mx240_dmeasure=99999;
       mx240_err=303;
    }

/*  Read measurement */
	else {
		Fmt(mx240_smeasure,"%s<%c%c%c%c%c%c",s[0],s[1],s[2],s[3],s[4],s[5]);
		Scan(mx240_smeasure,"%s>%f",&mx240_dmeasure);
    
    	/* set mx240_dmesure > 0 */
    	if (mx240_dmeasure==0.0) mx240_dmeasure=0.0;
    }

/*  Read unity and adjust mx240_dmeasure */
    mx240_coef=1;
    switch(s[6])
    	{
    	case 'n':
    		mx240_coef=1e-9;
    		break;
    	case 'u':
    	case '':
    		mx240_coef=1e-6;
    		break;
    	case 'm':
    		mx240_coef=1e-3;
    		break;
    	case 'k':
    		mx240_coef=1e+3;
    		break;
    	case 'M':
    		mx240_coef=1e+6;
    		break;
		}
    mx240_dmeasure=mx240_dmeasure*mx240_coef;

/*  Read Type of measurement (mx240_unit) */
	switch(s[7])
		{
		case 'A':
			if(s[10]=='D') mx240_unit=12;
			else if(s[12]=='+') mx240_unit=10;
			else mx240_unit=11;
			break;
		case 'V':
			if(s[8]=='A') mx240_unit=34;
			else if(s[10]=='D') mx240_unit=22;
			else if(s[12]=='+') mx240_unit=20;
			else mx240_unit=21;
			break;
		case 'W':
			if(s[8]=='h') mx240_unit=64;
			else mx240_unit=44;
			break;
		case 'v':
			mx240_unit=54;
			break;
		case 'O':
			mx240_unit=74;
			break;
		case 'C':
			if(s[9]=='A') mx240_unit=13;
			else mx240_unit=23;
			break;
		case 'F':
			mx240_unit=3;
			break;
		case 'c':
			if(s[10]=='L')
				mx240_unit=8;
			else
				mx240_unit=9;
			break;
		case 'H':
			mx240_unit=1;
			break;
		}

    return mx240_err;
	}

/*=========================================================================*/
/* Function: Device Closed                                                 */
/* Purpose:  This function checks to see if the module has been            */
/*           initialized.  If the device has not been opened, a -1 is      */
/*           returned, 0 otherwise.                                        */
/*=========================================================================*/
int mx240_device_closed (void)
{
    if (port == 0)  {
       mx240_err = 232;
       return -1;
    }
    return 0;
}
